/************************************************************************
 
 Module:      FlyVideo.cpp
 
 Project:     JTV
 
 Description: FlyVideo II/98 remote control driver. BUGGY!!!!!!
 
 Copyright (c) 1999 by Jaroslaw Kowalski <jaak@polbox.com>.
 All rights reserved.
 
 You may not use any piece of the following code for commercial purposes
 without written permission from the author.
 
 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
************************************************************************/

#include "stdafx.h"

#include <ut.h>
#include <btregs.h>

#include <jtvremsk.h>

class CFlyVideo : public CRemoteDriver
{
public:
	CFlyVideo(KeyDef *k, int count) : CRemoteDriver(k, count)
	{
	};

	void PollingThread()
	{
		IBT8x8Device *bt;

		if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
		{
			DWORD gpio_data, int_stat;
			DWORD gpio_dma_ctl;
			DWORD mask;
			DWORD gpio_reg_inp;
			DWORD gpio_out_en;

			log("Initializing FlyVideo Remote controller...\n");
			bt->ReadBT(0x10C, &gpio_dma_ctl);
			log("Old GPIO_DMA_CTL = %x\n", gpio_dma_ctl);

			bt->ReadBT(BT848_GPIO_REG_INP, &gpio_reg_inp);
			log("Old GPIO_REG_INP = %x\n", gpio_reg_inp);

			bt->ReadBT(BT848_GPIO_OUT_EN, &gpio_out_en);
			log("Old GPIO_OUT_EN = %x\n", gpio_out_en);

			bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
			log("Old GPIO_DATA = %x\n", gpio_data);

			bt->WriteBT(0x10C, (gpio_dma_ctl & 0xFF) | (3 << 14));

			bt->WriteBT(BT848_GPIO_REG_INP, 0x0);
			bt->WriteBT(BT848_GPIO_OUT_EN, 0xffffff);

			bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
			log("Old GPIO_DATA2 = %x\n", gpio_data);

			bt->ReadBT(0x100, &int_stat);
			bt->WriteBT(0x100, int_stat);

			bt->ReadBT(0x104, &mask);
			bt->WriteBT(0x104, mask | 512 | (1 << 23));

			bt->ReadBT(0x200, &gpio_data);
			log("gpio: %x\n", gpio_data);
			bt->WriteBT(0x200, (gpio_data | 0x8DFFF3) & ~0x1C00);
			bt->ReadBT(0x200, &gpio_data);
			log("gpio2: %x\n", gpio_data);
			last_tick = GetTickCount();

			while (S_OK == bt->WaitForInterrupt2(&int_stat, &gpio_data, NULL, hBreakEvent))
			//while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
        	{
				//static DWORD g, i;

				//bt->ReadBT(0x200, &gpio_data);
				//bt->ReadBT(0x100, &int_stat);
				//log("GPIO_DATA = %x\n", gpio_data);
				//Sleep(100);

				if (int_stat & 0x200)
				{
					if (!(int_stat & 0x100) || ((gpio_data & 0xFF) != 0))
					{
						bt->WriteBT(0x200, gpio_data | 0xFF);
						AddStroke(gpio_data & 0xFF);
					};
				}
			};

			log("Break event received.\n");

			bt->ReadBT(0x200, &gpio_data);
			bt->WriteBT(0x200, gpio_data & ~0xFF);

			bt->ReadBT(0x104, &mask);
			bt->WriteBT(0x104, mask & ~512);

			bt->ReadBT(0x10C, &gpio_dma_ctl);
			bt->WriteBT(0x10C, gpio_dma_ctl & ~(3<<14));

			bt->Release();
		};
	};
};

KeyDef keys[]=
{
	{ 0x00, "Fly_Power", "Power", },
	{ 0x80, "Fly_MouseMode", "MouseMode" },
    { 0x40, "Fly_TVFullScreen", },
    { 0x48, "Fly_ChannelUp", "ChannelUp" },
    { 0xc0, "Fly_1", "1", },
    { 0x20, "Fly_2", "2", },
    { 0xa0, "Fly_3", "3", },
    { 0xc8, "Fly_ChannelDown", "ChannelDown", },
    { 0xe0, "Fly_4", "4", },
    { 0x10, "Fly_5", "5", },
    { 0x90, "Fly_6", "6", },
    { 0x28, "Fly_VolumeUp", "VolumeUp" },
    { 0xd0, "Fly_7", "7", },
    { 0x30, "Fly_8", "8", },
    { 0xb0, "Fly_9", "9", },
    { 0xe8, "Fly_VolumeDown", "VolumeDown" },
    { 0xf8, "Fly_Cancel", },
    { 0xf0, "Fly_0", "0", },
    { 0x08, "Fly_Enter", "Enter" },
    { 0x60, "Fly_Recall", "Recall" },
    { 0x70, "Fly_AirCableTV", },
    { 0x88, "Fly_Video", },
    { 0x50, "Fly_Inc", "Inc"},
    { 0xa8, "Fly_Audio", },
    { 0x68, "Fly_Dec", "Dec"},
    { 0xb8, "Fly_Window", },
    { 0x98, "Fly_Down", },
    { 0x58, "Fly_Esc", },
    { 0x38, "Fly_TabRight", },
    { 0x78, "Fly_TabLeft", },
    { 0xd8, "Fly_Mute", "Mute" },
    { 0x18, "Fly_AVSource", },
};

extern "C"
{
	__declspec(dllexport) IJTVRemoteControl * __stdcall GetInterfacePtr()
	{
		return new CFlyVideo(keys, sizeof(keys) / sizeof(keys[0]));
	};
};
