/************************************************************************
 
 Module:      Leadtek.cpp
 
 Project:     JTV
 
 Description: LeadTek WinView remote control driver. UNCHECKED.
 
 Copyright (c) 1999 by Jaroslaw Kowalski <jaak@polbox.com>.
 All rights reserved.
 
 You may not use any piece of the following code for commercial purposes
 without written permission from the author.
 
 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
************************************************************************/
#include "stdafx.h"

#include <ut.h>
#include <jtvremsk.h>
#include <btregs.h>

KeyDef keys[]=
{
    { 0xA0, "Leadtek_Power", "Power", },
    { 0xA1, "Leadtek_Menu", "" },
    { 0xA2, "Leadtek_FM", "" },
    { 0xA3, "Leadtek_PCTV", "" },

    { 0xA4, "Leadtek_VolumeUp", "VolumeUp" },
    { 0xA5, "Leadtek_1", "1", },
    { 0xA6, "Leadtek_2", "2", },
    { 0xA7, "Leadtek_3", "3", },

    { 0xA8, "Leadtek_VolumeDown", "VolumeDown" },
    { 0xA9, "Leadtek_4", "4", },
    { 0xAA, "Leadtek_5", "5", },
    { 0xAB, "Leadtek_6", "6", },

    { 0xAC, "Leadtek_ChannelUp", "ChannelUp" },
    { 0xAD, "Leadtek_7", "7", },
    { 0xAE, "Leadtek_8", "8", },
    { 0xAF, "Leadtek_9", "9", },

    { 0xB0, "Leadtek_ChannelDown", "ChannelDown", },
    { 0xB1, "Leadtek_Return", "", },
    { 0xB2, "Leadtek_0", "0", },
    { 0xB3, "Leadtek_Display", "", },

    { 0xB4, "Leadtek_Mute", },
    { 0xB5, "Leadtek_TxtMix", },
    { 0xB6, "Leadtek_TxtStop", },
    { 0xB7, "Leadtek_TxtInfo", },

	{ 0xB8, "Leadtek_Red", },
    { 0xB9, "Leadtek_Green", },
    { 0xBA, "Leadtek_Yellow", },
    { 0xBB, "Leadtek_MTS", },

    { 0xBC, "Leadtek_TxtReveal", },
    { 0xBD, "Leadtek_Blue", },
    { 0xBE, "Leadtek_Video", },
    { 0xBF, "Leadtek_TxtCC", },
};


class CLeadtek : public CRemoteDriver
{
public:
	CLeadtek(KeyDef *k, int count) : CRemoteDriver(k, count)
	{
	};

	void PollingThread()
	{
		IBT8x8Device *bt;

		if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
		{
			DWORD gpio_data, int_stat;
			DWORD gpio_dma_ctl;
			DWORD mask;

			bt->ReadBT(0x10C, &gpio_dma_ctl);
			bt->WriteBT(0x10C, (gpio_dma_ctl & 0xFF) | (3 << 14));

			bt->WriteBT(BT848_GPIO_REG_INP, 0);

			bt->ReadBT(0x100, &int_stat);
			bt->WriteBT(0x100, int_stat);

			bt->ReadBT(0x104, &mask);
			bt->WriteBT(0x104, mask | 512 | (1 << 23));

			bt->ReadBT(0x200, &gpio_data);
			bt->WriteBT(0x200, gpio_data | 0xFF);

			last_tick = GetTickCount();

			while (bt->WaitForInterrupt2(&int_stat, &gpio_data, NULL, hBreakEvent))
			{
				if (int_stat & 0x200)
				{
					AddStroke((gpio_data >> 8) & 0xFF);
				}
			};

			log("Break event received.\n");

			bt->ReadBT(0x104, &mask);
			bt->WriteBT(0x104, mask & ~512);

			bt->ReadBT(0x10C, &gpio_dma_ctl);
			bt->WriteBT(0x10C, gpio_dma_ctl & ~(3<<14));


			bt->Release();
		};
	};
};
extern "C"
{
	__declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
	{
		return new CLeadtek(keys, sizeof(keys)/sizeof(keys[0]));
	};
};
